// Authors: Stefan Haustein (SH), Michael Kroll (MK)
// 2000-05-20 SH Changed to modal dialog
// 1999-08-19 SH initial release
// 1999-08-29 SH added TestDialog
// 1999-09-02 MK added TabbedPane
// 1999-11-07 MK version 0.11 released
// 1999-11-22 MK version 0.12 released
// 1999-11-28 MK version 0.2 released
// 1999-12-05 MK version 0.21 released
// 1999-12-15 MK version 0.22 released
// 2000-02-17 MK version 0.23 released
// 2000-03-04 MK version 0.24 released
// 2000-03-06 MK version 0.25 released
// 2000-05-22 MK version 0.60 released


import java.awt.*;
import java.awt.event.*;
import de.kawt.*;

class LayoutPanel extends Panel implements ActionListener {
    
    LayoutManager saveBorder;
    //TestDialog dialog; 
    Frame frame;
    
    LayoutPanel (Frame frame) {
	
	super (new BorderLayout ());
	this.frame = frame;
	//dialog = new TestDialog (frame);
	addButton ("North", "grid");
	addButton ("South", "border");
	addButton ("East", "flow");
	addButton ("West", "exit");
	addButton ("Center", "dialog");
	saveBorder = getLayout ();
	validate ();
    }

    void addButton (String where, String label) {
	Button b = new Button (label);
	add (where, b);
	b.addActionListener (this);
	b.setActionCommand (label);
    }
    
    public void actionPerformed (ActionEvent e) {
	String cmd = e.getActionCommand ();
	if (cmd.equals ("grid")) 
	    setLayout (new GridLayout (3, 2));
	else if (cmd.equals ("flow"))
	    setLayout (new FlowLayout ());
	else if (cmd.equals ("border"))
	    setLayout (saveBorder);
	else if (cmd.equals ("dialog")) {
	    if (OptionDialog.showConfirmDialog 
		(frame, "Do you like kAWT?", "Sample Dialog") 
		== OptionDialog.NO_OPTION)
		OptionDialog.showInputDialog 
		    (frame, "Why do you use it then?");
	}
	else if (cmd.equals ("exit")) 
	    System.exit (0);
	validate ();
    }
}

class MemPanel extends Canvas {

    public void paint (Graphics g) {

	g.drawString ("total: "+ Runtime.getRuntime().totalMemory(), 0,20);

	g.drawString ("free: "+Runtime.getRuntime().freeMemory (), 0, 40);
	Runtime.getRuntime().gc();
	g.drawString ("post gc: "+Runtime.getRuntime ().freeMemory (), 0, 60);
    }

}


class MiscPanel extends Panel implements ActionListener, ItemListener { 
    Frame frame;
    MiscPanel (Frame frame) {
	super (new GridLayout (0, 1));
	this.frame = frame;
	CheckboxGroup group = new CheckboxGroup ();
	add (new Checkbox ("RadioButton 1", group, false)); 
	add (new Checkbox ("RadioButton 2", group, false));
	Checkbox titlebox = new Checkbox ("Show Frame Title", true);
	titlebox.addItemListener (this);
	add (titlebox);
	add (new TextField ("Test"));
	Choice choice = new Choice ();
	choice.add ("red");
	choice.add ("green");
	choice.add ("blue");
	add (choice);
	addButton ("beep");
	addButton ("exit");
    }
    
    void addButton (String label) {
	Button b = new Button (label);
	add (b);
	b.addActionListener (this);
	b.setActionCommand (label);
    }
    
    public void itemStateChanged (ItemEvent ev) {
 	if (ev.getStateChange () == ev.SELECTED) 
	    frame.setTitle ("kAWT Sample v0.60");
	else 
	    frame.setTitle (null);
	frame.validate ();
    }
    
    public void actionPerformed (ActionEvent ev) {
	if (ev.getActionCommand ().equals ("beep")) {
	    Toolkit.getDefaultToolkit ().beep ();
	}
	else System.exit (0);
  }
}

class ScrollbarPanel extends Panel implements AdjustmentListener, ActionListener {
    Label label;
    Scrollbar horizontal;

    ScrollbarPanel (Frame frame) {
	super (new BorderLayout ());
	Button exitB = new Button ("exit");
	exitB.addActionListener (this);
	exitB.setActionCommand ("exit");
	add ("West", exitB);

	label = new Label ();
	horizontal = new Scrollbar (Scrollbar.HORIZONTAL);
	horizontal.addAdjustmentListener (this);
      
	java.awt.List list = new java.awt.List ();
	for (int i= 0; i < 20; i++)	
	    list.add ("item "+i);
	
	add ("North", label);
	add ("Center", list);
	add ("South", horizontal);
	adjustmentValueChanged (null);

	/*

	ScrollPane sp = new ScrollPane ();
	
	sp.setLayout (new GridLayout (0, 3));

	for (int i = 0; i < 35; i++)
	    sp.add (new Label (" *Field Nr. "+i+ "* "));

	add ("Center", sp);
	*/

	validate ();
    }
    
    public void adjustmentValueChanged (AdjustmentEvent ev) {
	label.setText ("Pos h: "+horizontal.getValue ());
    }
    
    public void actionPerformed (ActionEvent e) {
	String cmd = e.getActionCommand ();
	if (cmd.equals ("exit")) 
	    System.exit (0);
    }
}

class TestDialog extends Dialog implements ActionListener {
    TestDialog (Frame w) {
	super (w, "Sample Dialog");
	Panel panel = new Panel (new GridLayout (4, 1));
	add ("Center", new Label ("kAWT sample Dialog"));
	panel = new Panel ();
	Button button = new Button ("Close");
	button.addActionListener (this);
	panel.add (button);
	add ("South", panel);
	pack ();	
    }
    
    public void actionPerformed (ActionEvent e) {
	setVisible (false);
    }
}

class Closer extends WindowAdapter {
    public void windowClosing (WindowEvent e) {
	System.exit (0);
    }
}

public class KawtSample extends Frame {
    Dialog dialog;
    TabbedPane tabPane;

    KawtSample () {
	setTitle ("kAWT Sample v0.60");

	tabPane = new TabbedPane();
	tabPane.addTab ("Layout", new LayoutPanel (this));
	tabPane.addTab ("Misc", new MiscPanel (this));
	tabPane.addTab ("Scrollbar", new ScrollbarPanel (this));
	tabPane.addTab ("Mem", new MemPanel ());
	add ("Center", tabPane);
	
	addWindowListener (new Closer ());
	System.out.println ("bef pack");
	pack ();
	System.out.println ("aft pack");
    }
    
    public static void main (String [] argv) {
	new KawtSample ().show ();
	System.out.println ("still alive!");
    }
}






