/* Component.java Version 
 *
 * History:
 * 
 *  2000-05-20 SH improved repaint mechanism
 *  2000-05-18 SH visible check added to getGraphics
 *  2000-05-15 SH made getGraphics "ScrollPane-aware" 
 *  2000-05-01 SH background / foreground added
 *  2000-04-09 SH improved getGraphics 
 *  1999-11-22    Version 0.12 getSize () added
 *  1999-11-07    Version number 0.11 released
 *	          - internal variables width and height
 *		    changed to w and h
 *  1999-10-22    Version number 0.1 released
 *
 * Copyright 1999,2000 by Stefan Haustein & Michael Kroll
 * 
 * stefan.haustein@trantor.de
 * michael.kroll@trantor.de
 *
 * All rights reserved.
 *
 * NO WARRANTY
 *
 *  1. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO
 *  WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE
 *  LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
 *  HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT
 *  WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT
 *  NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE
 *  QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
 *  PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY
 *  SERVICING, REPAIR OR CORRECTION.
 * 
 *  2. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
 *  WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY
 *  MODIFY AND/OR REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE
 *  LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,
 *  INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
 *  INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
 *  DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU
 *  OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY
 *  OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN
 *  ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 *  END OF TERMS AND CONDITIONS */

package java.awt;

import java.util.Vector;
import java.awt.event.*;


/** Important Limitations:
 *
 *   <ul>
 *   <li>Only one listener of each type is allowed</li>
 *   <li>Many methods are still missing</li>
 *   <ul>
 *
 *   Many events are not passed through the processXXXEvent-Methods
 *   but delivered directly to the registered listener. 
 *
 */

public abstract class Component {

    int x;
    int y;
    int w;
    int h;

    boolean enabled = true;
    boolean visible = true;
    boolean hasFocus = false;
    boolean valid = false;
    
    Container parent;
    
    MouseListener mouseListener;
    KeyListener keyListener;

    
    //  Font font;
    Color background = Color.white;
    Color foreground = Color.black;
    
    public Component () {}
  
    public void addMouseListener (MouseListener l) {
	if (mouseListener != null)
	    throw new TooManyListenersException ();
	
	mouseListener =  l;
    }

    public void addKeyListener (KeyListener l) {
	if (keyListener != null)
	    throw new TooManyListenersException (); 
	
	keyListener =  l;
    }
    
    
    public Component findComponentAt (int px, int py) {
	return (visible && contains (px, py)) ? this : null; 
    }



    /** return null if not visible */
    
    public Point getLocationOnScreen () {
	
	// avoid recursion in order to save stack space...
	
	int sx = x;
	int sy = y;
	
	Component current = this;

	while (current.parent != null) {
	    current = current.parent;
	    if (!current.visible) return null;
	    sx += current.x;
	    sy += current.y;
	} 
	
	if (!(current instanceof Window)) return null;

	return new Point (sx, sy);
    }


    public Font getFont () {
	return Font.defaultFont;
    }

	   
    public FontMetrics getFontMetrics (Font font) {
	return new FontMetrics (font);
    }

    /* public Graphics getGraphics2 () {
	
	Point loc = getLocationOnScreen ();
	if (loc == null) return null; // null if not visible
	
	Graphics g = new Graphics ();
	g.translate (loc.x, loc.y);
	g.clipRect (0, 0, w, h);

	// substract all windows in front of the current:
	// getWindow, getFrame, ...

	Window window = _getWindow ();
	Frame frame = window.owner;
	
	int i0 = 0;

	if (frame != window) {
	    i0 = frame.windows.indexOf (window) + 1;
	    if (i0 == 0) return null;
	}
	    
	for (int i = i0; i < frame.windows.size (); i++) 
	    g._sub ((Window) frame.windows.elementAt (i));

	g.setColor (foreground);
	return g;
    }
    */	

    /*
    Rectangle getScreenRect (Rectangle r2) {
	
	if (!visible) return null;

	Rectangle r = new Rectangle (0, 0, w, h);
	if (r2 != null) r = r.intersection (r2);
	
	r.x += x;
	r.y += y;

	Component current = this;
	
	while (current.parent != null) {
	    current = current.parent;
	    if (!current.visible) return null;
	    if (current instanceof ScrollPane) {
		ScrollPane sp = (ScrollPane) current;
		Point p = sp.getLocationOnScreen ();
		if (p == null) return null;
		
		r = r.intersection 
		    (new Rectangle (p.x, p.y, sp.viewPortW, sp.viewPortH));
	    }
	    r.x += current.x;
	    r.y += current.y;
	}
	
	if (! (current instanceof Window)) return null;

	return r;
    }
    */    

    public Graphics getGraphics () {

	if (!visible) return null;

	int sx = x;
	int sy = y;

	Component current = this;
	Graphics g = new Graphics ();
	
	while (current.parent != null) {
	    current = current.parent;
	    if (!current.visible) return null;
	    if (current instanceof ScrollPane) {
		ScrollPane sp = (ScrollPane) current;
		Point p = sp.getLocationOnScreen ();
		if (p == null) return null;
		g.clipRect (p.x, p.y, sp.viewPortW, sp.viewPortH);
	    }
	    sx += current.x;
	    sy += current.y;
	}

	if (!(current instanceof Window)) return null;

	g.translate (sx, sy);
	g.clipRect (0, 0, w, h);
	
	// substract all windows in front of the current:
	// getWindow, getFrame, ...
	
	Window window = (Window) current;
	Frame frame = window.owner;
	
	int i0 = 0;

	if (frame != window) {
	    i0 = frame.windows.indexOf (window) + 1;
	    if (i0 == 0) return null;
	}
	    
	for (int i = i0; i < frame.windows.size (); i++) 
	    g._sub ((Window) frame.windows.elementAt (i));
	

	return g;
    }

	

    public Dimension getSize () {
	return new Dimension (w, h);
    }
    
    
    public int getX() {
	return x;
    }

    
    public int getY() {
	return y;
    }

    
    public int getWidth() {
	return w;
    }

    
    public int getHeight() {
	return h;
    }
    
    
    public Dimension getPreferredSize () {
	return getMinimumSize ();
    }
    
    
    public Dimension getMinimumSize () {
	return new Dimension (8, 8);
    }


    public boolean contains (int x, int y) {
	return x >= 0 && x <= w && y >= 0 && y <= h;
    }
  
    
    void doLayout () {  
	valid = true;
    }
    

    public Container getParent () {
	return parent;
    }


    public void invalidate () {
	valid = false;
	if (parent != null) parent.invalidate ();
    }
    

    public boolean isEnabled () {
	return enabled;
    }


    public boolean isVisible () {
	return visible;
    }

    public void setBackground (Color c) {
	if (!background.equals (c)){
	    background = c;
	    repaint ();
	}
    }

    
    public void setBounds (int x, int y, int w, int h) {
      this.x = x;
      this.y = y;
      this.w = w;
      this.h = h;
      //repaint ();
    }
    
    
    public void setEnabled (boolean b) {
	if (enabled != b) {
	    enabled = b;
	    repaint ();
	}
    }

    
    public void setForeground (Color c) {
	if (!foreground.equals (c)) {
	    foreground = c;
	    repaint ();
	}
    }

    public void setLocation (int x, int y) {
	this.x =  x;
	this.y =  y;
	//	repaint ();
    }
    

    public void setSize (int w, int h) {
	w =  w;
	h =  h;
    }
    

    public void setSize (Dimension d) {
	w =   d.width;
	h =  d.height;
    }
    

    public void paint (Graphics g){
    }

    
    public void paintAll (Graphics g) {
	if (visible && g != null) paint (g);
    }
    

    public void proxyRequestFocus (Component c) {
	if (parent != null) parent.proxyRequestFocus (c);
    }


    public void requestFocus () {
	if (parent != null) parent.proxyRequestFocus (this); 
    }


    public void update (Graphics g) {
	g.setColor (background);
	g.fillRect (0, 0, w, h);
	g.setColor (foreground);
	paintAll (g);
    }
    

    public void repaint (int x, int y, int w, int h) {
	
	SpotletWrapper.wrapper.addRepaint (this, new Rectangle (x, y, w, h));
    }

    
    public void repaint () {
	SpotletWrapper.wrapper.addRepaint (this, new Rectangle (0, 0, w, h));
    }


    /** Does not work because of the type cast Problem!
     *  All event sources call the specialized method 
     *  (e.g. processMouseEvent) directly
     */
    

    public void processMouseEvent (MouseEvent e) {
	if (mouseListener == null) return;
	
	switch (e.getID ()) {
	case MouseEvent.MOUSE_PRESSED: 
	    mouseListener.mousePressed (e); break;
	    
	case MouseEvent.MOUSE_RELEASED: 
	    mouseListener.mouseReleased (e); break;
	    
	case MouseEvent.MOUSE_CLICKED: 
	    mouseListener.mouseClicked (e); break;
	}
    }


    public void processKeyEvent (KeyEvent e) {
	if (keyListener == null) return;
	
	switch (e.getID ()) {
	case KeyEvent.KEY_PRESSED: keyListener.keyPressed (e); break;
	case KeyEvent.KEY_RELEASED: keyListener.keyReleased (e); break;
	case KeyEvent.KEY_TYPED: keyListener.keyTyped (e); break;
	}
    }


    public void setVisible (boolean v) {
	if (v != visible) {
	    visible = v;
	    invalidate ();
	    repaint ();
	}
    }


    public void validate () {
	valid = true;
    }

    /*
    void _validate () {
	valid = true;
    }
    */
    
    Window _getWindow () {
	Component w = this;
	while (w.parent != null) w=w.parent;

	return (Window) w;
    }
    
}
